'''
  Import the SVHN dataset
  Source: http://ufldl.stanford.edu/housenumbers/
  Description: Street View House Numbers

  ~~~ Important note ~~~
  Please cite the following paper when using or referencing the dataset:
  Yuval Netzer, Tao Wang, Adam Coates, Alessandro Bissacco, Bo Wu, Andrew Y. Ng Reading Digits in Natural Images with Unsupervised Feature Learning NIPS Workshop on Deep Learning and Unsupervised Feature Learning 2011. Retrieved from http://ufldl.stanford.edu/housenumbers/nips2011_housenumbers.pdf

'''

from keras.utils.data_utils import get_file
import numpy as np
from zipfile import ZipFile
from scipy import io as sio
import os

def load_data(path='svhn_matlab.npz', type='normal'):
  """Loads the SVHN dataset.
  # Arguments
      path: path where to cache the dataset locally
          (relative to ~/.keras/datasets).
      type: any of normal, extra (extra appends ~530K extra images for training)
  # Returns
      Tuple of Numpy arrays: `(input_train, target_train), (input_test, target_test)`.
  """
  path_train = get_file(f'{path}_train',
                    origin='http://ufldl.stanford.edu/housenumbers/train_32x32.mat')
  path_test = get_file(f'{path}_test',
                    origin='http://ufldl.stanford.edu/housenumbers/test_32x32.mat')

  # Load data from Matlab file.
  # Source: https://stackoverflow.com/a/53547262
  mat_train = sio.loadmat(path_train)
  mat_test = sio.loadmat(path_test)
  
  # Prepare training data
  input_train = mat_train['X']
  input_train = np.rollaxis(input_train, 3, 0)
  target_train = mat_train['y'].flatten()

  # Prepare testing data
  input_test = mat_test['X']
  input_test = np.rollaxis(input_test, 3, 0)
  target_test = mat_test['y'].flatten()

  # Append extra data, if required
  if type == 'extra':
    path_extra = get_file(f'{path}_extra',
                  origin='http://ufldl.stanford.edu/housenumbers/extra_32x32.mat')
    mat_extra = sio.loadmat(path_extra)
    input_extra = mat_extra['X']
    input_extra = np.rollaxis(input_extra, 3, 0)
    target_extra = mat_extra['y'].flatten()
    input_train = np.concatenate((input_extra, input_train))
    target_train = np.concatenate((target_extra, target_train))

  # Return data
  return (input_train, target_train), (input_test, target_test)


'''
  Import the KMNIST dataset
  Source: https://arxiv.org/abs/1812.01718
  Description: Japanese character MNIST dataset.

  ~~~ Important note ~~~
  Please cite the following paper when using or referencing the dataset:
  Clanuwat, T., Bober-Irizar, M., Kitamoto, A., Lamb, A., Yamamoto, K., & Ha, D. (2018). Deep learning for classical Japanese literature. arXiv preprint arXiv:1812.01718. Retrieved from https://arxiv.org/abs/1812.01718

'''

from keras.utils.data_utils import get_file
import numpy as np

def load_data(path='kmnist.npz', type='kmnist'):
  """Loads the KMNIST dataset.
  # Arguments
      path: path where to cache the dataset locally
          (relative to ~/.keras/datasets).
      type: any of kmnist, k49
  # Returns
      Tuple of Numpy arrays: `(input_train, target_train), (input_test, target_test)`.
  """
  # Load training images
  path_train = get_file(
    f'{path}_{type}_train_imgs',
    origin=f'http://codh.rois.ac.jp/kmnist/dataset/{type}/{type}-train-imgs.npz'
  )
  input_train = np.load(path_train)['arr_0']

  # Load training labels
  path_train_labels = get_file(
    f'{path}_{type}_train_labels',
    origin=f'http://codh.rois.ac.jp/kmnist/dataset/{type}/{type}-train-labels.npz'
  )
  target_train = np.load(path_train_labels)['arr_0']

  # Load testing images
  path_test = get_file(
    f'{path}_{type}_test_imgs',
    origin=f'http://codh.rois.ac.jp/kmnist/dataset/{type}/{type}-test-imgs.npz'
  )
  input_test = np.load(path_test)['arr_0']

  # Load testing labels
  path_test_labels = get_file(
    f'{path}_{type}_test_labels',
    origin=f'http://codh.rois.ac.jp/kmnist/dataset/{type}/{type}-test-labels.npz'
  )
  target_test = np.load(path_test_labels)['arr_0']

  # Return data
  return (input_train, target_train), (input_test, target_test)



'''
  Import the EMNIST dataset
  Source: https://www.nist.gov/itl/products-and-services/emnist-dataset
  Description: The EMNIST dataset is a set of handwritten character digits derived from the NIST Special Database 19  and converted to a 28x28 pixel image format and dataset structure that directly matches the MNIST dataset

  ~~~ Important note ~~~
  Please cite the following paper when using or referencing the dataset:
  Cohen, G., Afshar, S., Tapson, J., & van Schaik, A. (2017). EMNIST: an extension of MNIST to handwritten letters. Retrieved from http://arxiv.org/abs/1702.05373

'''

from keras.utils.data_utils import get_file
import numpy as np
from zipfile import ZipFile
from scipy import io as sio
import os

def load_data(path='emnist_matlab.npz', type='balanced'):
  """Loads the EMNIST dataset.
  # Arguments
      path: path where to cache the dataset locally
          (relative to ~/.keras/datasets).
      type: any of balanced, byclass, bymerge, digits, letters, mnist (defaults to balanced)
  # Returns
      Tuple of Numpy arrays: `(input_train, target_train), (input_test, target_test)`.
  """
  path = get_file(path,
                    origin='http://www.itl.nist.gov/iaui/vip/cs_links/EMNIST/matlab.zip')
  with ZipFile(path, 'r') as opened_zip:

    # Read file and temporarily store it
    file_name = f'./{type}.mat'
    f = open(file_name, 'wb')
    f.write(opened_zip.read(f'matlab/emnist-{type}.mat'))
    f.close()

    # Load data from Matlab file.
    # Source: https://stackoverflow.com/a/53547262
    mat = sio.loadmat(file_name)
    data = mat['dataset']
    input_train = data['train'][0,0]['images'][0,0]
    target_train = data['train'][0,0]['labels'][0,0].flatten()
    input_test = data['test'][0,0]['images'][0,0]
    target_test = data['test'][0,0]['labels'][0,0].flatten()

    # Remove data when loaded
    os.remove(file_name)

    # Reshape input data
    # Source: https://stackoverflow.com/a/53547262
    input_train = input_train.reshape((input_train.shape[0], 28, 28), order='F')
    input_test = input_test.reshape((input_test.shape[0], 28, 28), order='F')

    # Return data
    return (input_train, target_train), (input_test, target_test)


from distutils.core import setup
setup(
  name = 'extra_keras_datasets',
  packages = ['extra_keras_datasets'],
  version = '0.1.4',
  license='MIT',
  description = 'Extending the Keras Datasets module with extra ones.',
  long_description = 'Extending the Keras Datasets module with extra ones.',
  author = 'Christian Versloot',
  author_email = 'chris@machinecurve.com',
  url = 'https://github.com/christianversloot/extra_keras_datasets',
  download_url = 'https://github.com/christianversloot/extra_keras_datasets/archive/0.1.4.tar.gz',
  keywords = ['keras', 'datasets', 'machine learning'],
  install_requires=[
    'keras',
    'numpy',
    'scipy'
  ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)


'''
  Import the STL-10 dataset
  Source: https://cs.stanford.edu/~acoates/stl10/
  Description: The STL-10 dataset is an image recognition dataset for developing unsupervised feature learning, deep learning, self-taught learning algorithms.

  ~~~ Important note ~~~
  Please cite the following paper when using or referencing the dataset:
  Coates, A., Ng, A., & Lee, H. (2011, June). An analysis of single-layer networks in unsupervised feature learning. In Proceedings of the fourteenth international conference on artificial intelligence and statistics (pp. 215-223). Retrieved from http://cs.stanford.edu/~acoates/papers/coatesleeng_aistats_2011.pdf

'''

from keras.utils.data_utils import get_file
from scipy import io as sio
import shutil
import numpy as np

def load_data(path='stl10_matlab.tar.gz'):
  """Loads the STL-10 dataset.
  # Arguments
      path: path where to cache the dataset locally
          (relative to ~/.keras/datasets).
  # Returns
      Tuple of Numpy arrays: `(input_train, target_train), (input_test, target_test)`.
  """
  path = get_file(path,
                    origin='http://ai.stanford.edu/~acoates/stl10/stl10_matlab.tar.gz')

  # Temporarily extract .tar.gz in local path
  local_targz_path = './stl-10'
  shutil.unpack_archive(path, local_targz_path)

  # Load data from Matlab file
  # Source: https://stackoverflow.com/a/53547262
  train = sio.loadmat(f'{local_targz_path}/stl10_matlab/train.mat', )
  test = sio.loadmat(f'{local_targz_path}/stl10_matlab/test.mat')

  # Remove temporary file
  shutil.rmtree(local_targz_path, ignore_errors=True)

  # Define training data
  input_train = train['X'].reshape((-1, 3, 96, 96))
  input_train = np.transpose(input_train, (0, 3, 2, 1))
  target_train = train['y'].flatten()

  # Define testing data
  input_test = test['X'].reshape((-1, 3, 96, 96))
  input_test = np.transpose(input_test, (0, 3, 2, 1))
  target_test = test['y'].flatten()

  # Return data
  return (input_train, target_train), (input_test, target_test)